#include "validation.h"
#include <set>
#include <vector>
#include <map>
// In this file, you should validate the grammar of your input files.
// Any additional constraints mentioned in the problem statement should also be checked here.

// Please use the Validator class from the header for verifying the grammar.
// This automatically exits with code 43 (WA) when input parsing fails,
// and exits with code 42 when reading input and all tests succeeded.
// Please use v.WA("some ", "error ", "here ") to exit with code 43 manually.

// When your program crashes unexpectedly, or something weird is going on,
// please write some info to stderr and return anything different from 42 and 43.

int main(int argc, char **argv) {
	InputValidator v;
	int n = v.read_long_long(1, 100000);
	v.space();
	int m = v.read_long_long(0, 200000);
	v.space();
	int s = v.read_long_long(0, n - 1);
	v.space();
	int t = v.read_long_long(0, n - 1);
	v.newline();

	vector<bool> has_outgoing(n, false);

	for(int i = 0; i < m; i++) {
		int a = v.read_long_long(0, n - 1);
		v.space();
		int b = v.read_long_long(0, n - 1);
		v.space();
		int w = v.read_long_long(1, 10000);
		v.newline();

		if(a == b) v.WA("Direct loop.");

		has_outgoing[a] = true;
	}

	if(has_outgoing[t]) v.WA("Target node has outgoing edge.");
	for(int i = 0; i < n; i++) {
		if(i != t && !has_outgoing[i]) v.WA("Non-target node ", i, " has no outgoing edge.");
	}
}
