#include <algorithm>
#include <iostream>
#include <queue>
#include <vector>
using namespace std;
using ll = long long;
// Dijkstra
struct Edge{ int v; ll weight; };	// input edges
struct PQ{ ll d; int v; };			// distance and target
bool operator>(const PQ &l, const PQ &r){ return l.d > r.d; }
vector<ll> dijkstra(const vector<vector<Edge>> &edges, int s) {
	vector<ll> dist(edges.size(),1e18);
	priority_queue<PQ,vector<PQ>,greater<PQ>> pq;
	dist[s] = 0; pq.push({0, s});
	while (!pq.empty()) {
		auto d = pq.top().d; auto u = pq.top().v; pq.pop();
		if (d == dist[u])
			for(auto &e : edges[u]) if (dist[e.v] > d + e.weight){
				pq.push({dist[e.v] = d + e.weight, e.v});
			}
	}
	return dist;
}
// Dinic
using F = int;
struct S{
	const int v;			// neighbour
	F f;			// current flow
	F cap;	// capacity
	S(int v, F c) :
		v(v), f(0), cap(c) {}
};
struct FlowGraph : vector<S> {
	vector<vector<int>> g;
	FlowGraph(size_t n) : vector<S>(), g(n) {}
	F* add_edge(int u, int v, F c){ auto &t = *this;
		g[u].push_back(t.size());
		t.emplace_back(v, c);
		g[v].push_back(t.size());
		t.emplace_back(u, 0);
		return &t[t.size()-2].cap;
	}
	void reset(){
		for(auto &e : *this)
			e.f = 0;
	}
};
struct Dinic{
	FlowGraph &edges; int V,s,t;
	vector<int> l; vector<vector<int>::iterator> its; // levels and iterators
	Dinic(FlowGraph &edges, int s, int t) :
		edges(edges), V(edges.g.size()), s(s), t(t), l(V,-1), its(V) {}
	ll augment(int u, F c) { // we reuse the same iterators
		if (u == t) return c;
		for(auto &i = its[u]; i != edges.g[u].end(); i++){
			auto &e = edges[*i];
			if (e.cap > e.f && l[u] < l[e.v]) {
				auto d = augment(e.v, min(c, e.cap - e.f));
				if (d > 0) {
					e.f += d; edges[(*i)^1].f -= d; return d;
				}
			}	}
		return 0;
	}
	ll run() {
		ll flow = 0, f;
		while(true) {
			fill(l.begin(), l.end(),-1);
			l[s]=0; // recalculate the layers
			queue<int> q; q.push(s);
			while(!q.empty()){
				auto u = q.front(); q.pop();
				for(auto i : edges.g[u]){
					auto& e = edges[i];
					if(e.cap > e.f && l[e.v]<0)
						l[e.v] = l[u]+1, q.push(e.v);
				}
			}
			if (l[t] < 0) return flow;
			for (int u = 0; u < V; ++u) its[u] = edges.g[u].begin();
			while ((f = augment(s, 2e9)) > 0) flow += f;
		}	}
};
int main(){
	ios::sync_with_stdio(false);
	cin.tie(NULL);

	int n,m,a;
	cin >> n >> m >> a; // a <= 10
	vector<vector<Edge>> g(n);
	vector<long long> cs(n);
	long long people = 0;
	for(auto &c : cs) cin >> c, people+=c;
	while(m--){
		int u,v,w;
		cin >> u >> v >> w;
		--u, --v;
		g[u].push_back({v,w});
		g[v].push_back({u,w});
	}
	vector<Edge> as(a);
	for(auto &s : as)
		cin >> s.v >> s.weight, --s.v;
	vector<vector<ll>> dists(a);
	ll max_dist = 0;
	for(int i = 0; i < a; ++i){
		dists[i] = dijkstra(g, as[i].v);
		max_dist = max(max_dist, *max_element(dists[i].begin(), dists[i].end()));
	}

	vector<ll> unique_dists;
	unique_dists.reserve(n*a);
	for(const auto &x : dists)
		unique_dists.insert(unique_dists.end(), x.begin(), x.end());

	sort(unique_dists.begin(), unique_dists.end());
	unique_dists.erase(unique(unique_dists.begin(), unique_dists.end()), unique_dists.end());
	
	// BUILD THE FLOWGRAPH.
	FlowGraph f(a + n + 2);
	int s = a+n;
	int t = s+1;
	// s -> [0..n) -> [n..n+a) -> t
	// degrees: n, 1+a, 1+n, a
	// 1: cl[mask] capacity, all edges
	f.reserve(2*(n+a+n*a));

	for(int i = 0; i < n; ++i)
		if(cs[i] > 0)
			f.add_edge(s, i, cs[i]);
	// 2: infinite capacity, edges i -> n+j if dist[i][j] <= m
	vector<vector<int*>> cross_capacities(a, vector<int*>(n));
	for(int j = 0; j < a; ++j)
		for(int i = 0; i < n; ++i)
			if(cs[i] > 0)
				cross_capacities[j][i] = f.add_edge(i, n + j, 0);

	// 3: as[i] capacity, all edges
	for(int j = 0; j < a; ++j)
		f.add_edge(n+j, t, as[j].weight);

	auto l = unique_dists.begin(), h = unique_dists.end(); // The answer is somewhere in [l,h].

	ll last_m = 0;
	ll flow = 0;
	while(l < h){
		auto m = l + (h-l)/2; // Max distance m
		if(*m < last_m){
			f.reset();
			flow = 0;
		}
		last_m = *m;
		for(int j = 0; j < a; ++j)
			for(int i = 0; i < n; ++i)
				if(cs[i] > 0)
					*cross_capacities[j][i] = dists[j][i] <= *m ? 1e9 : 0;
		flow += Dinic(f, s, t).run();
		if(flow == people)
			h = m;
		else
			l = m + 1;
	}
	cout << *l << endl;
}
